// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'dart:io';
import 'dart:convert';

Future<void> runEm(String chars,
    {Duration sleep = const Duration(milliseconds: 45)}) async {
  for (var char in chars.split('')) {
    if (char == ' ') char = "Space";
    final result =
        await Process.run('tmux', 'send-keys -t asciinema $char'.split(' '));
    if (result.exitCode != 0) {
      print("Huh... there was a problem with: ${result.stderr}");
    }
    await Future.delayed(sleep);
  }
}

Future<void> enter() async {
  await Process.run('tmux', 'send-keys -t asciinema Enter'.split(' '));
}

Future<void> kill() async {
  await Process.run('tmux', 'send-keys -t asciinema C-d'.split(' '));
}

Future get anIota => Future.delayed(const Duration(milliseconds: 500));

class Delay {
  final Duration delay;

  Delay(this.delay);

  Future call(other) {
    print("await ${delay * other}");
    return Future.delayed(delay * other);
  }
}

final aSecond = Delay(const Duration(seconds: 1));

void main() async {
  print('loading a new window');
  var result = await Process.run(
      'tmux', 'new-window -n asciinema asciinema rec ../foo2.cast'.split(' '));
  if (result.exitCode != 0) {
    print("Huh... there was a problem with: ${result.stderr}");
  }
  await Future.delayed(const Duration(milliseconds: 300));


  print('sending some text');
  await runEm('# ProtoFu is here to help you manage your protobufs');
  await enter();
  await anIota;

  await runEm('# First, lets look at our project...');
  await enter();
  await anIota;
  await runEm('# It contains some pretty basic protos and a config file');
  await enter();
  await anIota;

  await runEm('tree');
  await enter();
  await aSecond(2);

  await runEm('# If we peek at the configuration file, you can see');
  await enter();
  await anIota;
  await runEm('# + You can customize the protoc and dart plugin versions');
  await enter();
  await anIota;
  await runEm('# + List your proto source directories');
  await enter();
  await anIota;
  await runEm('# + Link to external proto sources');
  await enter();
  await anIota;

  await runEm('batcat protofu.yaml');
  await enter();
  await aSecond(3);

  await runEm('# Lets look at a simple, self contained proto');
  await enter();
  await anIota;
  await runEm('batcat proto/test_simple.proto');
  await enter();
  await aSecond(2);

  await runEm('# A proto that imports third-party firestore?');
  await enter();
  await anIota;
  await runEm('batcat proto/test_external.proto');
  await enter();
  await aSecond(3);

  await runEm('# Now to compile to our lib/src/generated folder!');
  await enter();
  await anIota;
  await runEm(
      '# The first run downloads protoc, dart plugins, and third party');
  await enter();
  await anIota;

  await runEm('protofu');
  await enter();
  await anIota;
  await waitForProcess('protofu');

  await runEm(
      '# The consecutive runs are much faster');
  await enter();
  await anIota;

  await runEm('protofu');
  await enter();
  await anIota;
  await waitForProcess('protofu');

//tree -C lib | less -r
  await runEm(
      '# All protos compiled and ready for your editor!');
  await enter();
  await anIota;
  await Process.run('tmux', ['send-keys', '-t', 'asciinema', 'tree -C lib | less -r', 'Enter']);
  await aSecond(3);
  await Process.run('tmux', 'send-keys -t asciinema NPage'.split(' '));
  await aSecond(3);
  await runEm(
      'q');
  await anIota;

  await runEm(
      '# Have fun coding!');
  await enter();
  await anIota;

  print('sending done');
  await Future.delayed(const Duration(milliseconds: 300));
  await kill();
}

Future waitForProcess(String name) async {
  while (true) {
    final result = await Process.run('ps', ['a'], stdoutEncoding: utf8);
    if (result.stdout.contains(name)) {
      print('waiting for process to complete');
      await aSecond(0.5);
    } else {
      break;
    }
  }
}
